# -*- coding: utf-8 -*-
import unittest
import webapp2
from webapp2_restful.routes import ResourceRoute

__author__ = 'ekampf'


class PhotosHandler(webapp2.RequestHandler):
    pass


class PhotosCommentsHandler(webapp2.RequestHandler):
    pass


class TestResourceRoute(unittest.TestCase):
    def testNameIsPluralLowercase(self):
        self.assertEqual('photos', ResourceRoute('photo', PhotosHandler).name)
        self.assertEqual('photos', ResourceRoute('Photo', PhotosHandler).name)
        self.assertEqual('photos', ResourceRoute('Photos', PhotosHandler).name)
        self.assertEqual('users', ResourceRoute('User', PhotosHandler).name)
        self.assertEqual('users', ResourceRoute('Users', PhotosHandler).name)

    def testBasicRestRoutes(self):
        r = ResourceRoute('photos', PhotosHandler, actions=[('delete_all', 'POST')], member_actions=['thumb'])
        router = webapp2.Router([r])

        # GET all
        route_match, args, kwargs = router.match(self.__blank('/photos'))
        self.assertEqual(route_match.handler, PhotosHandler)
        self.assertEqual(route_match.handler_method, 'index')
        self.assertEqual(args, ())

        # POST
        route_match, args, kwargs = router.match(self.__blank('/photos', 'POST'))
        self.assertEqual(route_match.handler, PhotosHandler)
        self.assertEqual(route_match.handler_method, 'create')
        self.assertEqual(args, ())

        # GET a specific resource
        route_match, args, kwargs = router.match(self.__blank('/photos/123'))
        self.assertEqual(route_match.handler, PhotosHandler)
        self.assertEqual(route_match.handler_method, 'show')
        self.assertEqual(args, ())
        self.assertDictEqual(kwargs, dict(photo_id='123'))

        # DELETE a specific resource
        route_match, args, kwargs = router.match(self.__blank('/photos/123', 'DELETE'))
        self.assertEqual(route_match.handler, PhotosHandler)
        self.assertEqual(route_match.handler_method, 'destroy')
        self.assertEqual(args, ())
        self.assertDictEqual(kwargs, dict(photo_id='123'))

        # PUT a specific resource
        route_match, args, kwargs = router.match(self.__blank('/photos/123', 'PUT'))
        self.assertEqual(route_match.handler, PhotosHandler)
        self.assertEqual(route_match.handler_method, 'update')
        self.assertEqual(args, ())
        self.assertDictEqual(kwargs, dict(photo_id='123'))

        # Resource actions
        route_match, args, kwargs = router.match(self.__blank('/photos/delete_all', 'POST'))
        self.assertEqual(route_match.handler, PhotosHandler)
        self.assertEqual(route_match.handler_method, 'delete_all')
        self.assertEqual(args, ())
        self.assertDictEqual(kwargs, dict())

        # Resource member actions
        route_match, args, kwargs = router.match(self.__blank('/photos/123/thumb'))
        self.assertEqual(route_match.handler, PhotosHandler)
        self.assertEqual(route_match.handler_method, 'thumb')
        self.assertEqual(args, ())
        self.assertDictEqual(kwargs, dict(photo_id='123'))

    def testResourceWithMinusInName(self):
        r = ResourceRoute('ab-tests', PhotosHandler, actions=[('delete_all', 'POST')], member_actions=['thumb', 'scale-up'])
        router = webapp2.Router([r])

        # GET all
        route_match, args, kwargs = router.match(self.__blank('/ab-tests'))
        self.assertEqual(route_match.handler, PhotosHandler)
        self.assertEqual(route_match.handler_method, 'index')
        self.assertEqual(args, ())

        # POST
        route_match, args, kwargs = router.match(self.__blank('/ab-tests', 'POST'))
        self.assertEqual(route_match.handler, PhotosHandler)
        self.assertEqual(route_match.handler_method, 'create')
        self.assertEqual(args, ())

        # GET a specific resource
        route_match, args, kwargs = router.match(self.__blank('/ab-tests/123'))
        self.assertEqual(route_match.handler, PhotosHandler)
        self.assertEqual(route_match.handler_method, 'show')
        self.assertEqual(args, ())
        self.assertDictEqual(kwargs, dict(ab_test_id='123'))

        # DELETE a specific resource
        route_match, args, kwargs = router.match(self.__blank('/ab-tests/123', 'DELETE'))
        self.assertEqual(route_match.handler, PhotosHandler)
        self.assertEqual(route_match.handler_method, 'destroy')
        self.assertEqual(args, ())
        self.assertDictEqual(kwargs, dict(ab_test_id='123'))

        # PUT a specific resource
        route_match, args, kwargs = router.match(self.__blank('/ab-tests/123', 'PUT'))
        self.assertEqual(route_match.handler, PhotosHandler)
        self.assertEqual(route_match.handler_method, 'update')
        self.assertEqual(args, ())
        self.assertDictEqual(kwargs, dict(ab_test_id='123'))

        # Resource actions
        route_match, args, kwargs = router.match(self.__blank('/ab-tests/delete_all', 'POST'))
        self.assertEqual(route_match.handler, PhotosHandler)
        self.assertEqual(route_match.handler_method, 'delete_all')
        self.assertEqual(args, ())
        self.assertDictEqual(kwargs, dict())

        # Resource member actions
        route_match, args, kwargs = router.match(self.__blank('/ab-tests/123/thumb'))
        self.assertEqual(route_match.handler, PhotosHandler)
        self.assertEqual(route_match.handler_method, 'thumb')
        self.assertEqual(args, ())
        self.assertDictEqual(kwargs, dict(ab_test_id='123'))

        route_match, args, kwargs = router.match(self.__blank('/ab-tests/123/scale-up'))
        self.assertEqual(route_match.handler, PhotosHandler)
        self.assertEqual(route_match.handler_method, 'scale_up')
        self.assertEqual(args, ())
        self.assertDictEqual(kwargs, dict(ab_test_id='123'))

    def testSubResources(self):
        r = ResourceRoute('photos', PhotosHandler, sub_resources=[
            ResourceRoute('comments', PhotosCommentsHandler)
        ])
        router = webapp2.Router([r])

        # GET all
        route_match, args, kwargs = router.match(self.__blank('/photos/123/comments/comm774'))
        self.assertEqual(route_match.handler, PhotosCommentsHandler)
        self.assertEqual(route_match.handler_method, 'show')
        self.assertEqual(args, ())
        self.assertDictEqual(kwargs, dict(photo_id='123', comment_id='comm774'))

    def testUriFor(self):
        class Handler(webapp2.RequestHandler):
            def get(self, *args, **kwargs):
                pass

        app = webapp2.WSGIApplication([
            ResourceRoute('photos', PhotosHandler, actions=[('delete_all', 'POST')], member_actions=['thumb'], sub_resources=[
                ResourceRoute('comments', PhotosCommentsHandler)
            ])
        ])

        req = webapp2.Request.blank('http://localhost:80/')
        req.route = webapp2.Route('')
        req.route_args = tuple()
        req.route_kwargs = {}
        req.app = app
        app.set_globals(app=app, request=req)
        handler = Handler(req, webapp2.Response())
        handler.app = app

        for func in (handler.uri_for,):
            self.assertEqual(func('photos'), '/photos')
            self.assertEqual(func('photo', photo_id=123), '/photos/123')
            self.assertEqual(func('photo_comments', photo_id=123), '/photos/123/comments')
            self.assertEqual(func('photo_comment', photo_id=123, comment_id='bla'), '/photos/123/comments/bla')

    def __blank(self, path, method=None):
        req = webapp2.Request.blank(path)
        if method:
            req.method = method

        return req



# -*- coding: utf-8 -*-
import unittest

from webapp2_restful.reqparse.arguments import EmailArgument

__author__ = 'ekampf'


class TestEmailArgument(unittest.TestCase):
    def setUp(self):
        self.target = EmailArgument()

    def testEmailArgument_NoAtEmail_throwsValueError(self):
        with self.assertRaises(ValueError):
            self.target("invalid_email")

    def testEmailArgument_TwoWordsEmail_throwsValueError(self):
        with self.assertRaises(ValueError):
            self.target("eran@ekampf")

    def testEmailArgument_TwoAtEmail_throwsValueError(self):
        with self.assertRaises(ValueError):
            self.target("eran@@ekampf.com")

    def testEmailArgument_WithSpaceEmail_throwsValueError(self):
        with self.assertRaises(ValueError):
            self.target("er ran@@ekampf.com")

    def testRequestParser_validEmail_returnsValue(self):
        self.target("eran@ekampf.com")

    def testRequestParser_PointInEmail_returnsValue(self):
        self.target("eran.kampf@whatever.com")

    def testRequestParser_UpperCaseEmail_returnsValue(self):
        self.target("ERAN@EKAMPF.COM")



# -*- coding: utf-8 -*-
import json
import unittest
from mock import Mock, NonCallableMock
from webapp2 import Request

from webapp2_restful.reqparse import Argument, Namespace, RequestParser, InvalidParameterValue, MissingParameterError

__author__ = 'ekampf'


class TestRequestParserArgument(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    # region Namespace tests
    def test_namespace_existence(self):
        namespace = Namespace()
        namespace.foo = 'bar'
        namespace['bar'] = 'baz'
        self.assertEqual(namespace['foo'], 'bar')
        self.assertEqual(namespace.bar, 'baz')

    def test_namespace_missing(self):
        namespace = Namespace()
        self.assertRaises(AttributeError, lambda: namespace.spam)
        self.assertRaises(KeyError, lambda: namespace['eggs'])

    def test_namespace_configurability(self):
        self.assertTrue(isinstance(RequestParser().parse_args(None), Namespace))
        self.assertTrue(type(RequestParser(namespace_class=dict).parse_args(None)) is dict)
    # endregion

    # region Argument params
    def test_name(self):
        arg = Argument("foo")
        self.assertEqual(arg.name, "foo")

    def test_default(self):
        arg = Argument("foo", default=True)
        self.assertEquals(arg.default, True)

    def test_dest(self):
        arg = Argument("foo", dest="foobar")
        self.assertEqual(arg.dest, "foobar")

    def test_default_help(self):
        arg = Argument("foo")
        self.assertEqual(arg.help, None)

    def test_location_url(self):
        arg = Argument("foo", location="url")
        self.assertEquals(arg.location, "url")

    def test_location_url_list(self):
        arg = Argument("foo", location=["url"])
        self.assertEquals(arg.location, ["url"])

    def test_location_header(self):
        arg = Argument("foo", location="headers")
        self.assertEquals(arg.location, "headers")

    def test_location_json(self):
        arg = Argument("foo", location="json")
        self.assertEquals(arg.location, "json")

    def test_location_get_json(self):
        arg = Argument("foo", location="get_json")
        self.assertEquals(arg.location, "get_json")

    def test_location_header_list(self):
        arg = Argument("foo", location=["headers"])
        self.assertEquals(arg.location, ["headers"])

    def test_type(self):
        arg = Argument("foo", type=int)
        self.assertEquals(arg.type, int)

    def test_required(self):
        arg = Argument("foo", required=True)
        self.assertEqual(arg.required, True)

    def test_ignore(self):
        arg = Argument("foo", ignore=True)
        self.assertEqual(arg.ignore, True)

    def test_action_filter(self):
        arg = Argument("foo", action="filter")
        self.assertEqual(arg.action, u"filter")

    def test_action(self):
        arg = Argument("foo", action="append")
        self.assertEqual(arg.action, u"append")

    def test_choices(self):
        arg = Argument("foo", choices=[1, 2])
        self.assertEqual(arg.choices, [1, 2])

    def test_default_dest(self):
        arg = Argument("foo")
        self.assertEqual(arg.dest, None)

    def test_default_default(self):
        arg = Argument("foo")
        self.assertEqual(arg.default, None)

    def test_required_default(self):
        arg = Argument("foo")
        self.assertEqual(arg.required, False)

    def test_ignore_default(self):
        arg = Argument("foo")
        self.assertEqual(arg.ignore, False)

    def test_action_default(self):
        arg = Argument("foo")
        self.assertEqual(arg.action, u"store")

    def test_choices_default(self):
        arg = Argument("foo")
        self.assertEqual(len(arg.choices), 0)

    def test_source(self):
        req = Mock(['args', 'headers', 'values'])
        req.args = {'foo': 'bar'}
        req.headers = {'baz': 'bat'}
        arg = Argument('foo', location=['args'])
        self.assertEqual(arg.source(req), req.args)

        arg = Argument('foo', location=['headers'])
        self.assertEqual(arg.source(req), req.headers)

    def test_convert_default_type_with_null_input(self):
        """convert() should properly handle case where input is None"""
        arg = Argument('foo')
        self.assertEquals(arg.convert(None), None)

    def test_source_bad_location(self):
        req = Mock(['params'])
        arg = Argument('foo', location=['foo'])
        self.assertTrue(len(arg.source(req)) == 0) # yes, basically you don't find it

    def test_source_default_location(self):
        req = Mock(['params'])
        req._get_child_mock = lambda **kwargs: NonCallableMock(**kwargs)
        arg = Argument('foo')
        self.assertEqual(arg.source(req), req.params)

    def test_option_case_sensitive(self):
        arg = Argument("foo", choices=["bar", "baz"], case_sensitive=True)
        self.assertEqual(True, arg.case_sensitive)

        # Insensitive
        arg = Argument("foo", choices=["bar", "baz"], case_sensitive=False)
        self.assertEqual(False, arg.case_sensitive)

        # Default
        arg = Argument("foo", choices=["bar", "baz"])
        self.assertEqual(True, arg.case_sensitive)

    # endregion

    # region Request Parser
    def testRequestParser_parse_addArgument(self):
        req = Request.blank("/bubble?foo=barß")
        parser = RequestParser()
        parser.add_argument(Argument("foo"))

        args = parser.parse_args(req)
        self.assertEquals(args['foo'], u"barß")

    def testRequestParser_parse_unicode(self):
        req = Request.blank("/bubble?foo=barß")
        parser = RequestParser()
        parser.add_argument("foo")

        args = parser.parse_args(req)
        self.assertEquals(args['foo'], u"barß")

    def testRequestParser_parse_append_ignore(self):
        req = Request.blank("/bubble?foo=bar")

        parser = RequestParser()
        parser.add_argument("foo", ignore=True, type=int, action="append")

        args = parser.parse_args(req)
        self.assertEquals(args['foo'], None)

    def testRequestParser_parse_append_default(self):
        req = Request.blank("/bubble?")

        parser = RequestParser()
        parser.add_argument("foo", action="append"),

        args = parser.parse_args(req)
        self.assertEquals(args['foo'], None)

    def testRequestParser_parse_append(self):
        req = Request.blank("/bubble?foo=bar&foo=bat")

        parser = RequestParser()
        parser.add_argument("foo", action="append"),

        args = parser.parse_args(req)
        self.assertEquals(args['foo'], ["bar", "bat"])

    def testRequestParser_parse_append_single(self):
        req = Request.blank("/bubble?foo=bar")

        parser = RequestParser()
        parser.add_argument("foo", action="append"),

        args = parser.parse_args(req)
        self.assertEquals(args['foo'], ["bar"])

    def testRequestParser_parse_dest(self):
        req = Request.blank("/bubble?foo=bar")

        parser = RequestParser()
        parser.add_argument("foo", dest="bat")

        args = parser.parse_args(req)
        self.assertEquals(args['bat'], "bar")

    def testRequestParser_parse_required(self):
        req = Request.blank("/bubble")

        parser = RequestParser()
        parser.add_argument("foo", required=True)

        message = ''
        try:
            parser.parse_args(req)
        except MissingParameterError as e:
            message = e.message

        self.assertEquals(message, u'Missing required parameter foo in json or params')

        parser = RequestParser()
        parser.add_argument("bar", required=True, location=['values', 'cookies'])

        try:
            parser.parse_args(req)
        except Exception as e:
            message = e.message

        self.assertEquals(message, u"Missing required parameter bar in ['values', 'cookies']")

    def testRequestParser_default_append(self):
        req = Request.blank("/bubble")
        parser = RequestParser()
        parser.add_argument("foo", default="bar", action="append")

        args = parser.parse_args(req)

        self.assertEquals(args['foo'], "bar")

    def testRequestParser_default(self):
        req = Request.blank("/bubble")

        parser = RequestParser()
        parser.add_argument("foo", default="bar")

        args = parser.parse_args(req)
        self.assertEquals(args['foo'], "bar")

    def testRequestParser_callable_default(self):
        req = Request.blank("/bubble")

        parser = RequestParser()
        parser.add_argument("foo", default=lambda: "bar")

        args = parser.parse_args(req)
        self.assertEquals(args['foo'], "bar")

    def testRequestParser_none(self):
        req = Request.blank("/bubble")

        parser = RequestParser()
        parser.add_argument("foo")

        args = parser.parse_args(req)
        self.assertEquals(args['foo'], None)

    def testRequestParser_json_body(self):
        req = Request.blank('/', POST='{ "foo": "bar" }', environ={
            'CONTENT_TYPE': 'application/json;"',
        })
        parser = RequestParser()
        parser.add_argument("foo", type=lambda x: x*2, required=False)

        args = parser.parse_args(req)
        self.assertEqual(args['foo'], "barbar")

    def testRequestParser_type_is_callable(self):
        req = Request.blank("/bubble?foo=1")

        parser = RequestParser()
        parser.add_argument("foo", type=lambda x: x*2, required=False)

        args = parser.parse_args(req)
        self.assertEqual(args['foo'], "11")

    def testRequestParser_type_is_decimal(self):
        import decimal

        parser = RequestParser()
        parser.add_argument("foo", type=decimal.Decimal, location="json")

        req = Request.blank('/stam', POST=json.dumps(dict(foo="1.0025")), environ={
            'CONTENT_TYPE': 'application/json;"',
        })
        args = parser.parse_args(req)
        self.assertEquals(args['foo'], decimal.Decimal("1.0025"))

    def testRequestParser_type_is_bool(self):
        import decimal

        parser = RequestParser()
        parser.add_argument("foo", type=bool)

        args = parser.parse_args(Request.blank('/stam?foo=true'))
        self.assertEquals(args['foo'], True)

        args = parser.parse_args(Request.blank('/stam?foo=True'))
        self.assertEquals(args['foo'], True)

        args = parser.parse_args(Request.blank('/stam?foo=t'))
        self.assertEquals(args['foo'], True)

        args = parser.parse_args(Request.blank('/stam?foo=1'))
        self.assertEquals(args['foo'], True)

        args = parser.parse_args(Request.blank('/stam?foo=f'))
        self.assertEquals(args['foo'], False)

        args = parser.parse_args(Request.blank('/stam?foo=0'))
        self.assertEquals(args['foo'], False)

        args = parser.parse_args(Request.blank('/stam?foo=false'))
        self.assertEquals(args['foo'], False)

        args = parser.parse_args(Request.blank('/stam?foo=False'))
        self.assertEquals(args['foo'], False)

    def testRequestParser_noParams_returnsNone(self):
        req = Request.blank('/')

        parser = RequestParser()
        parser.add_argument("foo")

        args = parser.parse_args(req)
        self.assertEqual(args['foo'], None)

    def testRequestParser_choices_correct(self):
        req = Request.blank("/bubble?foo=bat")

        parser = RequestParser()
        parser.add_argument("foo", choices=["bat"]),

        args = parser.parse_args(req)
        self.assertEquals(args['foo'], "bat")

    def testRequestParser_choices(self):
        req = Request.blank("/bubble?foo=bar")

        parser = RequestParser()
        parser.add_argument("foo", choices=["bat"]),

        self.assertRaises(InvalidParameterValue, lambda: parser.parse_args(req))

    def testRequestParser_choices_sensitive(self):
        req = Request.blank("/bubble?foo=BAT")

        parser = RequestParser()
        parser.add_argument("foo", choices=["bat"], case_sensitive=True),

        self.assertRaises(InvalidParameterValue, lambda: parser.parse_args(req))

    def testRequestParser_choices_insensitive(self):
        req = Request.blank("/bubble?foo=BAT")

        parser = RequestParser()
        parser.add_argument("foo", choices=["bat"], case_sensitive=False),

        args = parser.parse_args(req)
        self.assertEquals('bat', args.get('foo'))

        # both choices and args are case_insensitive
        req = Request.blank("/bubble?foo=bat")

        parser = RequestParser()
        parser.add_argument("foo", choices=["BAT"], case_sensitive=False),

        args = parser.parse_args(req)
        self.assertEquals('bat', args.get('foo'))

    def testRequestParser_choices_types_int(self):
        parser = RequestParser()
        parser.add_argument("foo", type=int, choices=[1, 2, 3], location='json')

        req = Request.blank('/stam', POST=json.dumps(dict(foo=5)), environ={
            'CONTENT_TYPE': 'application/json;"',
        })

        self.assertRaises(InvalidParameterValue, parser.parse_args, req)

    def test_int_range_choice_types(self):
        parser = RequestParser()
        parser.add_argument("foo", type=int, choices=range(100), location='json')

        req = Request.blank('/stam', POST=json.dumps(dict(foo=101)), environ={
            'CONTENT_TYPE': 'application/json;"',
        })
        self.assertRaises(InvalidParameterValue, parser.parse_args, req)

        req = Request.blank('/stam', POST=json.dumps(dict(foo=99)), environ={
            'CONTENT_TYPE': 'application/json;"',
        })
        args = parser.parse_args(req)
        self.assertEqual(99, args.get('foo'))

    def testRequestParser_ignore(self):
        req = Request.blank("/bubble?foo=bar")

        parser = RequestParser()
        parser.add_argument("foo", type=int, ignore=True)

        args = parser.parse_args(req)
        self.assertEquals(args['foo'], None)

    def testRequestParser_noParamsWithDefault_returnsDefault(self):
        req = Request.blank('/')

        parser = RequestParser()
        parser.add_argument('foo', default='faa')

        args = parser.parse_args(req)
        self.assertEqual(args['foo'], 'faa')

    def testRequestParser_copy(self):
        req = Request.blank("/bubble?foo=101&bar=baz")
        parser = RequestParser()
        foo_arg = Argument('foo', type=int)
        parser.args.append(foo_arg)
        parser_copy = parser.copy()

        # Deepcopy should create a clone of the argument object instead of
        # copying a reference to the new args list
        self.assertFalse(foo_arg in parser_copy.args)

        # Args added to new parser should not be added to the original
        bar_arg = Argument('bar')
        parser_copy.args.append(bar_arg)
        self.assertFalse(bar_arg in parser.args)

        args = parser_copy.parse_args(req)
        self.assertEquals(args['foo'], 101)
        self.assertEquals(args['bar'], u'baz')

    def testRequestParser_replace_argument(self):
        req = Request.blank("/bubble?foo=baz")
        parser = RequestParser()
        parser.add_argument('foo', type=int)
        parser_copy = parser.copy()
        parser_copy.replace_argument('foo', type=str)

        args = parser_copy.parse_args(req)
        self.assertEquals(args['foo'], u'baz')

    # endregion



# -*- coding: utf-8 -*-
import unittest

from webapp2_restful.reqparse import Argument
from webapp2_restful.reqparse.arguments import SafeStringArgument

__author__ = 'ekampf'


class TestStringArgument(unittest.TestCase):
    def setUp(self):
        self.target = Argument('test', type=SafeStringArgument())

    def testUnicodeValue(self):
        ustr = u'M·A·C'

        result = self.target.convert(ustr)
        self.assertEqual(result, ustr.encode('ascii', 'ignore'))

    def testRegularString(self):
        s = "test123"
        result = self.target.convert(s)
        self.assertEqual(result, s)



from webapp2 import Route
from webapp2_extras.routes import MultiRoute

from inflection import singularize, pluralize

__author__ = 'ekampf'

# pylint:disable=C0326,R0902

class ResourceRoute(MultiRoute):
    ALL_REST_ACTIONS = ['index', 'create', 'show', 'update', 'destroy']

    def __init__(self, name, handler,
                 actions=None,
                 member_actions=None,
                 sub_resources=None,
                 only=None,
                 without=None,
                 path_prefix='',
                 name_prefix=''):

        """
        Defines routes for a RESTful resource.

        Example:

        >>> APP = WondermallWSGIApplication([
        >>>         ResourceRoute('photos', PhotosHandler, actions=[('delete_all', 'POST')], member_actions=['thumb'], sub_resources=[
        >>>             ResourceRoute('comments', PhotosCommentsHandler, only=['index', 'show'])
        >>>         ]
        >>> ])

        Creates the following routes:

          HTTP            PATH                                       Handler Method                   Name (uri_for)
        ----------------------------------------------------------------------------------------------------------------
        GET    - /photos                                - PhotosHandler.index(self)               - 'photos'
        POST   - /photos                                - PhotosHandler.create(self)              - 'photos'
        GET    - /photos/:photo_id                      - PhotosHandler.show(self, photo_id)      - 'photo'
        PUT    - /photos/:photo_id                      - PhotosHandler.update(self, photo_id)    - 'photo'
        DELETE - /photos/:photo_id                      - PhotosHandler.destroy(self, photo_id)   - 'photo'

        POST   - /photos/delete_all                     - PhotosHandler.delete_all                - 'photos_delete_all'
        GET    - /photos/:photo_id/thumb                - PhotosHandler.thumb(self, photo_id)     - 'photo_thumb'

        GET    - /photos/:photo_id/comments             - PhotosCommentsHandler.index(self)       - 'photo_comments'
        GET    - /photos/:photo_id/comments/:comment_id - PhotosCommentsHandler.show(self,        - 'photo_comment'
                                                                                     photo_id,
                                                                                     comment_id)


        :param name: The resource name. Has to be plural ('People', 'Posts', 'Users', ...)
        :param handler: The handler class to handle the resource
        :param actions: Additional actions that apply on the resource collection (all the resources of this type). An array of tuples (action_name, http_method) or just a name for GET actions
        :param member_actions: Additional action that apply to resource members (specific resource entities). An array of tuples (action_name, http_method) or just a name for GET actions
        :param sub_resources: An array of sub resources
        :param only: Specifies only to create specific REST routes. An array of strings with these possible values: ['index', 'create', 'show', 'update', 'destroy']
        :param without: Omit specific REST routes. An array of strings with these possible values: ['index', 'create', 'show', 'update', 'destroy']
        :param path_prefix: A path to prefix all teh resource's paths with. For example given '/api/v1' the resource's paths will be '/api/v1/photos' etc.
        :param name_prefix: A prefix to use for path name. For example: given 'api' the named routes would be 'api_photos', 'api_photo', etc.

        :type name: str
        :type handler: webapp2.RequestHandler
        :type actions: list[(str, str)|str]
        :type member_actions: list[(str, str)|str]
        :type sub_resources: list[ResourceRoute]

        :rtype: ResourceRoute
        """
        self.name = pluralize(name.lower())
        self.__handler = handler
        self.__actions = actions or []
        self.__member_actions = member_actions or []
        self.__sub_resources = sub_resources or []

        self.__path_prefix = path_prefix
        self.__name_prefix = name_prefix + '_' if name_prefix else ''

        self.supported_actions = only or self.ALL_REST_ACTIONS
        if without:
            self.supported_actions = [action for action in self.supported_actions if action not in without]

        # Make sure actions are (name, httml_method) tuples
        self.__actions        = [a if isinstance(a, tuple) else (str(a), 'GET') for a in self.__actions]
        self.__member_actions = [a if isinstance(a, tuple) else (str(a), 'GET') for a in self.__member_actions]

        routes = self.__get_routes()
        super(ResourceRoute, self).__init__(routes)

    def __get_routes(self):
        singular_name = singularize(self.name).replace('-', '_')
        resources_path = "%s/%s" % (self.__path_prefix, self.name)
        resource_path  = "%s/%s/<%s_id>" % (self.__path_prefix, self.name, singular_name)

        routes = []

        if "index" in self.supported_actions:
            routes.append(Route(resources_path, handler=self.__handler, methods=['GET', 'OPTIONS'], handler_method='index', name=self.__name_prefix + self.name))

        if "create" in self.supported_actions:
            routes.append(Route(resources_path, handler=self.__handler, methods=['POST', 'OPTIONS'], handler_method='create', name=self.__name_prefix + self.name))

        if "show" in self.supported_actions:
            routes.append(Route(resource_path,  handler=self.__handler, methods=['GET', 'OPTIONS'], handler_method='show', name=self.__name_prefix + singular_name))

        if "update" in self.supported_actions:
            routes.append(Route(resource_path,  handler=self.__handler, methods=['PUT', 'OPTIONS'], handler_method='update', name=self.__name_prefix + singular_name))

        if "destroy" in self.supported_actions:
            routes.append(Route(resource_path,  handler=self.__handler, methods=['DELETE', 'OPTIONS'], handler_method='destroy', name=self.__name_prefix + singular_name))

        for action_name, http_method in self.__actions:
            routes.append(Route(resources_path + '/' + action_name, handler=self.__handler, handler_method=action_name.replace('-', '_'), methods=['OPTIONS', http_method], name="%s_%s" % (self.__name_prefix + self.name, action_name)))

        for action_name, http_method in self.__member_actions:
            routes.append(Route(resource_path + '/' + action_name, handler=self.__handler, handler_method=action_name.replace('-', '_'), methods=['OPTIONS', http_method], name="%s_%s" % (self.__name_prefix + singular_name, action_name)))

        for sub in self.__sub_resources:
            for sub_route in sub.get_routes():
                routes.append(Route(resource_path + sub_route.template, handler=sub_route.handler, handler_method=sub_route.handler_method, methods=sub_route.methods, name="%s_%s" % (self.__name_prefix + singular_name, sub_route.name)))

        return routes



# -*- coding: utf-8 -*-
import base64
import unittest

from webapp2_restful.reqparse.arguments import Base64StringArgument

__author__ = 'ekampf'


class TestBase64Argument(unittest.TestCase):
    def setUp(self):
        self.target = Base64StringArgument()

    def testEmailArgument_invalidBase64_raisesValueError(self):
        with self.assertRaises(ValueError):
            expected = "I'm a value"
            self.target("abc"+base64.urlsafe_b64encode(expected))

    def testEmailArgument_validBase64_returnsDecodedValue(self):
        expected = "I'm a value"
        result = self.target(base64.urlsafe_b64encode(expected))
        self.assertEqual(expected, result)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

import codecs
import os
import re

root_dir = os.path.abspath(os.path.dirname(__file__))

def get_build_number():
    fname = 'build.info'
    if os.path.isfile(fname):
        with open(fname) as f:
            build_number = f.read()
            build_number = re.sub("[^a-z0-9]+","", build_number, flags=re.IGNORECASE)
            return '.' + build_number

    return ''

def get_version(package_name):
    build_number = get_build_number()

    version_re = re.compile(r"^__version__ = [\"']([\w_.-]+)[\"']$")
    package_components = package_name.split('.')
    init_path = os.path.join(root_dir, *(package_components + ['__init__.py']))
    with codecs.open(init_path, 'r', 'utf-8') as f:
        for line in f:
            match = version_re.match(line[:-1])
            if match:
                return match.groups()[0]+build_number

    return '0.1.0' + build_number

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read().replace('.. :changelog:', '')

requirements = [
    'webapp2==2.5.2',
    'webob==1.2.3',
    'jsonschema==2.3.0',
    'inflection==0.3.1'
]

test_requirements = [
    'mock==1.1.2'
]

setup(
    name='webapp2_restful',
    version=get_version('webapp2_restful'),
    description="A Request parsing interface designed to provide simple and uniform access to any variable on the webapp2.Request object in Webapp2",
    long_description=readme + '\n\n' + history,
    author="Eran Kampf",
    author_email='eran@ekampf.com',
    url='https://github.com/ekampf/webapp2_restful',
    packages=[
        'webapp2_restful',
    ],
    package_dir={'webapp2_restful': 'webapp2_restful'},
    include_package_data=True,
    setup_requires=[
        'setuptools-lint',
    ],
    install_requires=requirements,
    license="BSD",
    zip_safe=False,
    keywords='webapp2_restful',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
    ],
    test_suite='tests',
    tests_require=test_requirements
)



# -*- coding: utf-8 -*-
import unittest

import json, jsonschema
from webapp2_restful.reqparse.arguments import JSONArgument

__author__ = 'ekampf'


# noinspection PyProtectedMember
class TestJSONArgument(unittest.TestCase):
    def setUp(self):
        self.target = JSONArgument()

    def test_none_returnsNone(self):
        self.assertIsNone(self.target(None))

    def test_validJSON_validObjectReturned(self):
        expected_obj = {'id': '1234', 'name': 'Jenna Jameson'}
        obj_json = json.dumps(expected_obj)

        actual_obj = self.target(obj_json)

        self.assertEqual(expected_obj, actual_obj)

    def test_invalidJSON_valueErrorRaised(self):
        obj_json = "{'id': , 'name': 'Jenna Jameson'}"
        with self.assertRaises(ValueError):
            self.target(obj_json)

    def test_validJSONWithMatchingSchema_validObjectReturned(self):
        expected_obj = {'id': '1234', 'name': 'Jenna Jameson'}
        obj_json = json.dumps(expected_obj)

        schema = {
            'type': 'object',
            'properties': {
                'id': {'type': 'string', 'minLength': 1},
                'name': {'type': 'string', 'minLength': 1}
            }
        }

        actual_obj = self.target(obj_json)
        self.assertEqual(expected_obj, actual_obj)

    def test_validJSONWithUnmatchingSchema_validationError(self):
        expected_obj = {'id': '1234', 'name': 'Jenna Jameson'}
        obj_json = json.dumps(expected_obj)

        schema = {
            'type': 'object',
            'properties': {
                'id': {'type': 'number'},
                'name': {'type': 'string', 'minLength': 1}
            }
        }

        target = JSONArgument(schema=schema)
        with self.assertRaises(jsonschema.ValidationError):
            target(obj_json)



# -*- coding: utf-8 -*-
import unittest

from webapp2_restful.reqparse.arguments import DateStringArgument

__author__ = 'ekampf'


class TestDateStringArgument(unittest.TestCase):
    def test_default_format_valid(self):
        target = DateStringArgument()
        dt = target('2015-07-16 08:34:57700140')
        self.assertEqual(dt.year, 2015)
        self.assertEqual(dt.month, 7)
        self.assertEqual(dt.day, 16)
        self.assertEqual(dt.hour, 8)
        self.assertEqual(dt.minute, 34)
